
import java.io.*;
import java.security.*;

public class ComputeSHA {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		//Verify that a file name was provided.
		if(args.length > 0)
		{
			FileInputStream fileReader = null;
			int c;								//Will keep the byte read from the file.
			byte[] hash = null;					//Will receive the hash value after applying the algorithm.
			byte[] byteRead = new byte[1];		//A 1-value array for reading each byte from file.

			try
			{
				fileReader = new FileInputStream(args[0]);						//Attempts to open file.
				MessageDigest algorithm = MessageDigest.getInstance("SHA");		//Attempts to get an instance of the algorithm.
				
				algorithm.reset();
				
				c = fileReader.read(byteRead);
				while(c != -1)	
				{												//Appends the message digest process
					algorithm.update(byteRead);					//byte by byte from the file.
					c = fileReader.read(byteRead);
				}
				
				hash = algorithm.digest();				//Apply algorithm over bytes from file.	
				
				System.out.println(toHexadecimalString(hash));	//Print hexadecimal string equivalent to hash bytes.
			}
			catch(IOException ioexception)
			{
				System.out.println("Failed to open file " + args[0]);
			}
			catch(NoSuchAlgorithmException nsaexception)
			{
				System.out.println("Failed to initialize algorithm for encryption");
			}
			finally
			{
				closeInputStream(fileReader);	//Attempts to close the file input stream.
			}
		}
		else
			System.out.println("Please provide the file name.");
	}
	
	/* Function to close a given file input stream */
	private static void closeInputStream(FileInputStream fis)
	{
		try
		{
			if(fis != null)	//Check for not null pointers.
				fis.close();	//Attempts to close the input stream signaled by fis.
		}
		catch(IOException e)
		{
			System.out.println("Could not close the input stream: " + e.getMessage());
		}
	}
	
	/* Function to transform the digest byte array into a hexadecimal equivalent string */
	private static String toHexadecimalString(byte[] byteStr)
	{
		StringBuffer buffer = new StringBuffer();	//It will be collecting the chars.
		for(int I=0; I < byteStr.length; I++)
		{
			String h = Integer.toHexString(0xff & byteStr[I]);
			if(h.length() == 1)						//A digit?
				buffer.append('0');
			buffer.append(h);						//Grow the string byte by byte.
		}
		return buffer.toString();		
	}

}
