/***************************************************

Autosuggest control class to handle dynamic suggesting.

***************************************************/

function AutoSuggestControl(objTextBox, objProvider)
{
   this.layer = null;			//The suggestions div.
   this.cur = -1;			//Store current suggestion.
   this.textBox = objTextBox;		//Store references.
   this.provider = objProvider;
   
   this.backField = null;		//To show current suggestion.
   this.ruler = null;			//Div to measure the length in pixels of current input.
   
   this.init();				//Initialize the control.
}

// Function to select a range of the text of the textbox.
AutoSuggestControl.prototype.selectRange = function(start, nlength) 
{
   if(this.textBox.createTextRange)	//Is it for IE?
   {
      var objRange = this.textBox.createTextRange();
      objRange.moveStart("character", start);					//Move to start initial position.
      objRange.moveEnd("character", nlength - this.textBox.value.length);	//Set the last char to select.
      objRange.select();							//Select the characters.
   }
   else 
   {
      if(this.textBox.setSelectionRange)					//Is it for Firefox?
      {
         this.textBox.setSelectionRange(start, nlength);
      }
   }

   this.textBox.focus();							//Focus the textbox.
}

// Function to perform type ahead in the textbox.
//AutoSuggestControl.prototype.typeAhead = function(sSuggestion)
//{
//   if(this.textBox.createTextRange || this.textBox.setSelectionRange)		//Implement typeAhead only if supported.
//   {
//      if(sSuggestion.toLowerCase().indexOf(this.textBox.value.toLowerCase()) == 0)//Does the suggestion contain what the user has typed?
//      {
//         var len = this.textBox.value.length;
//         this.textBox.value = sSuggestion;			//Fill in the textbox with a suggest.
//         this.selectRange(len, sSuggestion.length);		//Select untyped (still) text.
//      }
//   }
//}

// Function to perform type ahead in the textbox.
AutoSuggestControl.prototype.typeAhead = function(sSuggestion)
{

   if(sSuggestion.length > 0 && sSuggestion.toLowerCase().indexOf(this.textBox.value.toLowerCase()) == 0)//Does the suggestion contain what the user has typed?
   {
      var len = this.textBox.value.length;
      
      this.ruler.innerHTML = this.textBox.value;
      
      if(sSuggestion.length > len)		//Check that the suggestion is larger than current text.
       this.ruler.innerHTML = this.ruler.innerHTML + "<b>" + sSuggestion.substring(len) + "</b>";	//Build the background text.
      
      if(this.ruler.offsetWidth < this.textBox.offsetWidth)	//Verify if it is possible to show background text.
      {
         this.backField.innerHTML = this.ruler.innerHTML;
      }
      else
         this.backField.innerHTML = "";				//Clean background text.
   }
   else
      this.backField.innerHTML = "";				//Clean background text.
}

// Function to perform autosuggest. It must receive an array of suggestions.
AutoSuggestControl.prototype.autosuggest = function(aSuggestions, bTypeAhead)
{
   if(aSuggestions.length > 0) 		//Only if there are suggestions to show.
   {
      if(bTypeAhead)
         this.typeAhead(aSuggestions[0]);	//Pick the very first suggestion.
      this.showSuggestions(aSuggestions);	//Show the div layer if there are suggestions to display.
   }
   else
      this.hideSuggestions();			//Hide suggestions if the aSuggestions array is empty.
      
   this.cur = -1;				//Reset current suggestion.
}

// Function to handle the keyboard events on the textbox.
AutoSuggestControl.prototype.handleKeyUp = function(oEvent)
{
   var iKeyCode = oEvent.keyCode;

   if(iKeyCode == 8 || iKeyCode == 46)			//Deactivate type ahead with backspace and delete.
   {
      this.provider.requestSuggestions(this, true);
   } 
   else
   {
      if(iKeyCode < 32 || (iKeyCode >= 33 && iKeyCode <= 46) || (iKeyCode >= 112 && iKeyCode <= 123)) 
      {
	  //ignore
      } 
      else 
      {
	 this.provider.requestSuggestions(this, true);	//Pass a pointer to provider since he will invoke autosuggest,
      }
   }							//and determine is activate or deactivate type ahead.
   
   //Lastly, double-check if the search box is empty.
   if(this.textBox.value.length == 0)
      this.hideSuggestions();
}

// Function to initialize the AutoSuggestControl object.
AutoSuggestControl.prototype.init = function()
{
   var oThis = this;					//Save reference pointer for anonymous function.
   
   this.textBox.onkeyup = function(oEvent)		//Assign the onkeyup event to the textbox.
   {
      if(!oEvent) 					//Detect IE event handler.
      {
          oEvent = window.event;
      }
      oThis.handleKeyUp(oEvent);
   };
   
   this.textBox.onkeydown = function(oEvent)		//Define the onkeydown event for the searchfield.
   {
      if(!oEvent)
      {
          oEvent = window.event;
      }

      oThis.handleKeyDown(oEvent);
   };
   
   this.textBox.onblur = function()			//Hide the layer when it loses focus.
   {
      oThis.hideSuggestions();
   };

   this.createDropDown();				//Create the layer.
   this.createBackField();				//Create the back field and ruler.
}

// Function to hide the suggestions layer.
AutoSuggestControl.prototype.hideSuggestions = function()
{
   this.layer.style.visibility = "hidden";
   this.typeAhead("");					//Erase any previous suggestion in the backField.
}

// Function to highlight the current suggestion on the suggestions layer.
AutoSuggestControl.prototype.highlightSuggestion = function(oSuggestionNode)
{
   for(var I=0; I < this.layer.childNodes.length; I++)	//Update all div nodes of the layer.
   {
      var oNode = this.layer.childNodes[I];		//Found it? Change its class.
      if(oNode == oSuggestionNode)
      {
         oNode.className = "current";
      }
      else 
      {
         if(oNode.className == "current")		//Normalize the previously selected child node.
	 {
            oNode.className = "";
         }
      }
   }
}

// Function to initialize the backField and the Ruler.
AutoSuggestControl.prototype.createBackField = function()
{
   this.backField = document.getElementById("backField");	//Get the back field for showing current suggestion.
   this.backField.style.left = this.getLeft() + "px";	//Locate it in the web page.
   this.backField.style.top = this.getTop() + "px";
   this.backField.style.visibility= "visible";
   this.textBox.style.backgroundColor = "transparent";	//Make textbox bgcolor transparent.
   
   this.ruler = document.createElement("span");		//Create the ruler.
   this.ruler.id = "ruler";				//Link it to its css properties.
   document.body.appendChild(this.ruler);		//Although invisible, it is attached to the document.
}

// Function to create the dropdown div layer for showing suggestions.
AutoSuggestControl.prototype.createDropDown = function()
{
   this.layer = document.createElement("div");		//Creates the parent div.
   this.layer.className = "suggestions";		//Link parent div to class suggestions.
   this.layer.style.visibility = "hidden";		//Hide layer.
   this.layer.style.width = (this.textBox.offsetWidth-2) + "px";	//Assign appropriate width.
   document.body.appendChild(this.layer);		//Attach layer to document.

   var oThis = this;

   this.layer.onmousedown = this.layer.onmouseup =
   this.layer.onmouseover = function(oEvent)		//Anonymous function to handle mouse events on layer.
   {
      oEvent = oEvent || window.event;			//Find out which event object to use.
      oTarget = oEvent.target || oEvent.srcElement;	//Layer's child div where the mouse event happened.

      if(oEvent.type == "mousedown")
      {
         oThis.textBox.value = oTarget.firstChild.nodeValue;	//Set value of searchfiled when a suggestions is clicked.
         oThis.hideSuggestions();
      } 
      else 
      {
         if(oEvent.type == "mouseover")			//Highlights the current suggestion pointed by the mouse.
	 {
            oThis.highlightSuggestion(oTarget);
         } 
	 else 
	 {
            oThis.textBox.focus();			//Mouseup yields to change the focus to the searchfield.
         }
      }
   };
}

// Function to compute the left offset of the searchfield.
AutoSuggestControl.prototype.getLeft = function()
{
   var oNode = this.textBox;
   var iLeft = 0;

   while(oNode.tagName != "BODY") 	//Traverse the hierarchy of containers.
   {
      iLeft += oNode.offsetLeft;
      oNode = oNode.offsetParent;
   }

   return iLeft;
}

// Function to compute the top offset of the searchfield.
AutoSuggestControl.prototype.getTop = function()
{
   var oNode = this.textBox;
   var iTop = 0;

   while(oNode.tagName != "BODY") 	//Traverse the hierarchy of containers.
   {
       iTop += oNode.offsetTop;
       oNode = oNode.offsetParent;
   }

   return iTop;
}

// Function to show the layer of suggestions.
AutoSuggestControl.prototype.showSuggestions = function(aSuggestions)
{
   var oDiv = null;
   this.layer.innerHTML = "";

   for(var I=0; I < aSuggestions.length; I++)	//Create and push the individual suggestions in the div layer.
   {
      oDiv = document.createElement("div");
      oDiv.appendChild(document.createTextNode(aSuggestions[I]));
      this.layer.appendChild(oDiv);
   }

   this.layer.style.left = this.getLeft() + "px";
   this.layer.style.top = (this.getTop()+this.textBox.offsetHeight) + "px";
   this.layer.style.visibility = "visible";
}

// Function to control going to next suggestion in list.
AutoSuggestControl.prototype.nextSuggestion = function()
{
   var cSuggestionNodes = this.layer.childNodes;

   if(cSuggestionNodes.length > 0)
   {
      if(this.cur < cSuggestionNodes.length-1)	//Space to move forward?
         this.cur ++;
      else
         this.cur = 0;
      
      var oNode = cSuggestionNodes[this.cur];
      this.highlightSuggestion(oNode);
      this.textBox.value = oNode.firstChild.nodeValue;	//Substitute current text in search field.
      this.typeAhead("");				//Erase any previous suggestion.
   }
}

// Function to control going to previous suggestion in list.
AutoSuggestControl.prototype.previousSuggestion = function()
{
   var cSuggestionNodes = this.layer.childNodes;

   if(cSuggestionNodes.length > 0)
   {
      if(this.cur > 0)				//Space to move backwards?
         this.cur --;
      else
         this.cur = cSuggestionNodes.length-1;
      
      var oNode = cSuggestionNodes[this.cur];
      this.highlightSuggestion(oNode);
      this.textBox.value = oNode.firstChild.nodeValue;	//Substitute current text in search field.
      this.typeAhead("");				//Erase any previous suggestion.      
   }
}

// Function to define the handler for the arrow keys and enter.
AutoSuggestControl.prototype.handleKeyDown = function(oEvent)
{
   switch(oEvent.keyCode)
   {
      case 38: //up arrow
         this.previousSuggestion();
         break;
      case 40: //down arrow
         this.nextSuggestion();
         break;
      case 13: //enter
         this.hideSuggestions();
         break;
   }
}

