package edu.ucla.cs.cs144;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.SQLWarning;

import org.apache.lucene.document.Field;
import org.apache.lucene.document.Document;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.IndexWriter;

import java.io.*;
import java.util.Vector;

public class Indexer 
{
	private IndexWriter itemIndexWriter = null;
    
    /** Creates a new instance of Indexer */
    public Indexer() {
    }    
   
    /* Function to close the open index writers */
    public void closeIndexWriters() throws IOException 
    {
    	if(itemIndexWriter != null)	//Do not close it if it does not exist.
    	{
    		itemIndexWriter.close();
        }
    }
 
    /* Function to reload the indices */
    public void rebuildIndexes() 
    {
        Connection conn = null;

        //Create a connection to the database to retrieve Items from MySQL.
        try 
        {
        	conn = DbManager.getConnection(true);

        	//Erase any pre-existing indexes and get new ones.
        	String directory = System.getenv("LUCENE_INDEX") + "/itemIndex";	//For item.
        	itemIndexWriter = new IndexWriter(directory, new StandardAnalyzer(), true);
        	
        	//Create a channel for communicating with the CS144 database.
        	Statement statement = conn.createStatement();
        
        	//Fetch all necessary fields from the item relation to create the lucene index.
        	String query = "SELECT item.itemID AS itemID, name, description, categories " +
        		"FROM item, (SELECT itemID, GROUP_CONCAT(category SEPARATOR ' ') AS categories FROM itemCategory GROUP BY itemID) AS A " +
        		"WHERE A.itemID=item.itemID;";
        	ResultSet rsItem = statement.executeQuery(query);
        	SQLWarning warning;
        
        	//Index each tuple using Lucene.
        	System.out.println("* Creating index, please wait...");
        	int I = 0;
        	while(rsItem.next())
        	{
        		/*warning = rsItem.getWarnings();
        		if(warning != null) 
        			System.out.println("Message: " + warning.getMessage());*/
        		
        		indexItemTuple(rsItem.getString("itemID"),
        				rsItem.getString("name"),
        				rsItem.getString("description"), 
        				rsItem.getString("categories"));
        		
        		I++;
        	}
        	statement.close();
        	System.out.println("* " + I + " item tuples have been indexed.");

        	//Close index writer.
        	closeIndexWriters();
        
        	// close the database connection
        	conn.close();
        } 
        catch (SQLException ex) 
        {
        	System.err.println(ex);
        }
        catch(IOException e)
        {
        	System.err.println("Could not create indices correctly: " + e.getMessage());
        }
    }
    
    /* Function to fill in the index with items data and categories for current item tuple*/
    public void indexItemTuple(String itemID, String name, String description, String categories) 
    {
    	Document doc = new Document();			//Create a new document for this tuple.
    	
    	try
    	{
    		doc.add(new Field("itemID", itemID, Field.Store.YES, Field.Index.NO));
            doc.add(new Field("name", name, Field.Store.NO, Field.Index.TOKENIZED));
            doc.add(new Field("description", description, Field.Store.NO, Field.Index.TOKENIZED));
            doc.add(new Field("categories", categories, Field.Store.NO, Field.Index.TOKENIZED));
            String content = name + " " + description  + " " + categories;
            doc.add(new Field("content", content, Field.Store.NO, Field.Index.TOKENIZED));	//Composite field.
            
            itemIndexWriter.addDocument(doc);	//Write the document to the index.
    	}
    	catch(IOException ioe)
    	{
    		System.err.println("Could not write item index correctly: " + ioe.getMessage());
    	}
    }

    /* Main function */
    public static void main(String agrs[]) 
    {
        Indexer idx = new Indexer();
        idx.rebuildIndexes();
    }
}
