/* CS144
 *
 * Parser skeleton for processing item-???.xml files. Must be compiled in
 * JDK 1.5 or above.
 *
 * Instructions:
 *
 * This program processes all files passed on the command line (to parse
 * an entire diectory, type "java MyParser myFiles/*.xml" at the shell).
 *
 * At the point noted below, an individual XML file has been parsed into a
 * DOM Document node. You should fill in code to process the node. Java's
 * interface for the Document Object Model (DOM) is in package
 * org.w3c.dom. The documentation is available online at
 *
 * http://java.sun.com/j2se/1.5.0/docs/api/index.html
 *
 * A tutorial of Java's XML Parsing can be found at:
 *
 * http://java.sun.com/webservices/jaxp/
 *
 * Some auxiliary methods have been written for you. You may find them
 * useful.
 */

package edu.ucla.cs.cs144;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Attr;					//Added to support attributes.
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ErrorHandler;


class MyParser {
    
	/* File writer objects for creating the files used by MySQL-s LOAD command */
	static PrintWriter itemSchemaFile;
    static PrintWriter userLocSchemaFile;
    static PrintWriter userCountrySchemaFile;
    static PrintWriter itemCategorySchemaFile;
    static PrintWriter userRatingSchemaFile;
    static PrintWriter itemBuyPriceSchemaFile;
    static PrintWriter bidInfoSchemaFile;
	
    static final String columnSeparator = "|*|";
    static DocumentBuilder builder;
    
    static final String[] typeName = {
	"none",
	"Element",
	"Attr",
	"Text",
	"CDATA",
	"EntityRef",
	"Entity",
	"ProcInstr",
	"Comment",
	"Document",
	"DocType",
	"DocFragment",
	"Notation",
    };
    
    static class MyErrorHandler implements ErrorHandler {
        
        public void warning(SAXParseException exception)
        throws SAXException {
            fatalError(exception);
        }
        
        public void error(SAXParseException exception)
        throws SAXException {
            fatalError(exception);
        }
        
        public void fatalError(SAXParseException exception)
        throws SAXException {
            exception.printStackTrace();
            System.out.println("There should be no errors " +
                               "in the supplied XML files.");
            System.exit(3);
        }
        
    }
    
    /* Non-recursive (NR) version of Node.getElementsByTagName(...)
     */
    static Element[] getElementsByTagNameNR(Element e, String tagName) {
        Vector< Element > elements = new Vector< Element >();
        Node child = e.getFirstChild();
        while (child != null) {
            if (child instanceof Element && child.getNodeName().equals(tagName))
            {
                elements.add( (Element)child );
            }
            child = child.getNextSibling();
        }
        Element[] result = new Element[elements.size()];
        elements.copyInto(result);
        return result;
    }
    
    /* Returns the first subelement of e matching the given tagName, or
     * null if one does not exist. NR means Non-Recursive.
     */
    static Element getElementByTagNameNR(Element e, String tagName) {
        Node child = e.getFirstChild();
        while (child != null) {
            if (child instanceof Element && child.getNodeName().equals(tagName))
                return (Element) child;
            child = child.getNextSibling();
        }
        return null;
    }
    
    /* Returns the text associated with the given element (which must have
     * type #PCDATA) as child, or "" if it contains no text.
     */
    static String getElementText(Element e) {
        if (e.getChildNodes().getLength() == 1) {
            Text elementText = (Text) e.getFirstChild();
            return elementText.getNodeValue();
        }
        else
            return "";
    }
    
    /* Returns the text (#PCDATA) associated with the first subelement X
     * of e with the given tagName. If no such X exists or X contains no
     * text, "" is returned. NR means Non-Recursive.
     */
    static String getElementTextByTagNameNR(Element e, String tagName) {
        Element elem = getElementByTagNameNR(e, tagName);
        if (elem != null)
            return getElementText(elem);
        else
            return "";
    }
    
    /* Returns the amount (in XXXXX.xx format) denoted by a money-string
     * like $3,453.23. Returns the input if the input is an empty string.
     */
    static String strip(String money) {
        if (money.equals(""))
            return money;
        else {
            double am = 0.0;
            NumberFormat nf = NumberFormat.getCurrencyInstance(Locale.US);
            try { am = nf.parse(money).doubleValue(); }
            catch (ParseException e) {
                System.out.println("This method should work for all " +
                                   "money values you find in our data.");
                System.exit(20);
            }
            nf.setGroupingUsed(false);
            return nf.format(am).substring(1);
        }
    }
    
    /* Process one items-???.xml file.
     */
    static void processFile(File xmlFile) {
        Document doc = null;
        try {
            doc = builder.parse(xmlFile);
        }
        catch (IOException e) {
            e.printStackTrace();
            System.exit(3);
        }
        catch (SAXException e) {
            System.out.println("Parsing error on file " + xmlFile);
            System.out.println("  (not supposed to happen with supplied XML files)");
            e.printStackTrace();
            System.exit(3);
        }
        
        /* At this point 'doc' contains a DOM representation of an 'Items' XML
         * file. Use doc.getDocumentElement() to get the root Element. */
        System.out.println("Successfully parsed - " + xmlFile);
        
        /**************************** Processing of data starts ******************************/

        Element rootNode = doc.getDocumentElement();					//Get the root node.
        Element[] itemNodes = getElementsByTagNameNR(rootNode, "Item");	//Get the Item-children of the root node Items.
        int I;
        
        for(I=0; I<itemNodes.length; I++)		//Traverse the list of items.
        {	
        	////////////Create the schemas and vectors for data that will be encountered//////////
        	ItemSchema itemSchema = new ItemSchema();
        	ItemBuyPriceSchema itemBuyPriceSchema;					//To be determined after checking existence of buy price.
        	Vector <ItemCategorySchema> itemCategoryVector = new Vector <ItemCategorySchema>();
        	ItemCategorySchema[] itemCategorySchema = null;			//Determined by size of its vector.
        	Vector <UserLocSchema> userLocVector = new Vector <UserLocSchema>();
        	UserLocSchema[] userLocSchema = null;					//Depends on number of users with defined location.
        	Vector <UserCountrySchema> userCountryVector = new Vector <UserCountrySchema>();
        	UserCountrySchema[] userCountrySchema = null;			//Depends on number of users with defined country.
        	Vector <UserRatingSchema> userRatingVector = new Vector <UserRatingSchema>();
        	UserRatingSchema[] userRatingSchema = null;				//Depends on number of users, including the seller.
        	Vector <BidInfoSchema> bidInfoVector = new Vector <BidInfoSchema>();
        	BidInfoSchema[] bidInfoSchema = null;					//Depends on size of its vector.
        	//////////////////////////////////////////////////////////////////////////////////////
        	
        	//Item's ID.
        	Attr attr = itemNodes[I].getAttributeNode("ItemID");
        	itemSchema.itemID = attr.getValue();
        	
        	//Item's name.
        	itemSchema.name = getElementTextByTagNameNR(itemNodes[I], "Name");
        	
        	//Item's description.
        	String desc = getElementTextByTagNameNR(itemNodes[I], "Description");
        	if(desc.length() > 4000)		//Check length to not exceed 4000 chars.
        		desc = desc.substring(0, 4000);
        	itemSchema.description = desc;
        	
        	//Item's first bid.
        	itemSchema.firstBid = strip(getElementTextByTagNameNR(itemNodes[I], "First_Bid"));
        	
        	//Item's current value.
        	itemSchema.currently = strip(getElementTextByTagNameNR(itemNodes[I], "Currently"));
        	
        	//Item's auction start timestamp.
        	itemSchema.started = parseDate(getElementTextByTagNameNR(itemNodes[I], "Started"));
        	
        	//Item's auction ends timestamp.
        	itemSchema.ends = parseDate(getElementTextByTagNameNR(itemNodes[I], "Ends"));
        	
        	//Item's number of bids.
        	itemSchema.numberOfBids = getElementTextByTagNameNR(itemNodes[I], "Number_of_Bids");
        	
        	//Item's seller.
        	itemSchema.sellerID = getElementByTagNameNR(itemNodes[I], "Seller").getAttributeNode("UserID").getValue();
        	
        	//Check for null values to send a warning.
    		if(itemSchema.name.length()==0 || itemSchema.description.length()==0 || itemSchema.firstBid.length()==0 || itemSchema.currently.length()==0 || itemSchema.started.length()==0 || itemSchema.ends.length()==0 || itemSchema.numberOfBids.length()==0 || itemSchema.sellerID.length()==0)
    			System.out.println("Item: "+itemSchema.itemID+" has null values");

        	///////////////// Fill in seller values into the seller's loc schema //////////////////
        	UserLocSchema sellerLoc = new UserLocSchema();
        	
        	//Seller's ID.
        	sellerLoc.userID = itemSchema.sellerID;
        	
        	//Seller's location.
        	sellerLoc.location = getElementTextByTagNameNR(itemNodes[I], "Location");
        	
        	//Check for null values to send a warning.
    		if(sellerLoc.userID.length()==0 || sellerLoc.location.length()==0)
    			System.out.println("Seller: "+sellerLoc.userID+" has null location");
        	
        	//---Attach seller's data into the user's loc vector.
        	userLocVector.add(sellerLoc);
        	
        	///////////////// Fill in seller values into the seller's country schema //////////////////
        	UserCountrySchema sellerCountry = new UserCountrySchema();
        	
        	//Seller's ID.
        	sellerCountry.userID = itemSchema.sellerID;
        	
        	//Seller's Country.
        	sellerCountry.country = getElementTextByTagNameNR(itemNodes[I], "Country");
        	
        	//Check for null values to send a warning.
    		if(sellerCountry.userID.length()==0 || sellerCountry.country.length()==0)
    			System.out.println("Seller: "+sellerCountry.userID+" has null country");
        	
        	//---Attach seller's data into the user's loc vector.
        	userCountryVector.add(sellerCountry);
        	
        	///////////////// Fill in seller values into the seller's rating schema //////////////
        	UserRatingSchema sellerRating = new UserRatingSchema();
        	
        	//Seller's ID.
        	sellerRating.userID = sellerLoc.userID;
        	
        	//Seller's rating.
        	sellerRating.rating = getElementByTagNameNR(itemNodes[I], "Seller").getAttributeNode("Rating").getValue();
        	
        	//Check for null values to send a warning.
    		if(sellerRating.userID.length()==0 || sellerRating.rating.length()==0)
    			System.out.println("Seller: "+sellerRating.userID+" has null rating");
        	
        	//---Attach seller's data into the user's rating vector.
        	userRatingVector.add(sellerRating);
        	
        	////////////////// Fill in the buy price (if exists) for a given item ///////////////////
        	Element buyPriceElt = getElementByTagNameNR(itemNodes[I], "Buy_Price");
        	if(buyPriceElt != null)		//Element exists.
        	{
        		itemBuyPriceSchema = new ItemBuyPriceSchema();
        		itemBuyPriceSchema.itemID = itemSchema.itemID;
        		itemBuyPriceSchema.buyPrice = strip(getElementText(buyPriceElt));
        		
        		//Check for null values to send a warning.
        		if(itemBuyPriceSchema.buyPrice.length()==0)
        			System.out.println("Buy price for item: "+itemBuyPriceSchema.itemID+" has null value");
        	}
        	else
        		itemBuyPriceSchema = null;
        	
        	///////////////////////// Fill in the categories of the item /////////////////////////
        	Element[] categories = getElementsByTagNameNR(itemNodes[I], "Category");
        	for(int J=0; J < categories.length; J++)
        	{
        		ItemCategorySchema itemCategory = new ItemCategorySchema();
        		itemCategory.itemID = itemSchema.itemID;
        		itemCategory.category = getElementText(categories[J]);
        		itemCategoryVector.add(itemCategory);
        		
        		//Check for null values to send a warning.
        		if(itemCategory.category.length()==0)
        			System.out.println("Category for item: "+itemCategory.itemID+" has null values");
        	}
        	
        	//////////////////////// Fill in data about the bids ///////////////////////
        	Element bids = getElementByTagNameNR(itemNodes[I], "Bids");		//New root.
        	Element[] bidNodes = getElementsByTagNameNR(bids, "Bid");
        	for(int J=0; J < bidNodes.length; J++)
        	{
        		//////////////////////// Fill in bidder's loc schema //////////////////////
        		Element bidder = getElementByTagNameNR(bidNodes[J], "Bidder");	//Get a reference.
        		if(getElementByTagNameNR(bidder, "Location") != null)			//Location exists?
        		{
        			UserLocSchema bidderLoc = new UserLocSchema();
        			
        			//Bidder id.
        			bidderLoc.userID = bidder.getAttributeNode("UserID").getValue();
        			
        			//Bidder location.
        			bidderLoc.location = getElementTextByTagNameNR(bidder, "Location");
        			
        			//Check for null values to send a warning.
            		if(bidderLoc.userID.length()==0 || bidderLoc.location.length()==0)
            			System.out.println("Bidder: "+bidderLoc.userID+" has null location");
        			
        			//---Attach this user to the vector of userloc.
        			userLocVector.add(bidderLoc);
        		}
        		
        		////////////////////////Fill in bidder's country schema //////////////////////
        		if(getElementByTagNameNR(bidder, "Country") != null)			//Country exists?
        		{
        			UserCountrySchema bidderCountry = new UserCountrySchema();
        			
        			//Bidder id.
        			bidderCountry.userID = bidder.getAttributeNode("UserID").getValue();
        			
        			//Bidder country.
        			bidderCountry.country = getElementTextByTagNameNR(bidder, "Country");
        			
        			//Check for null values to send a warning.
            		if(bidderCountry.userID.length()==0 || bidderCountry.country.length()==0)
            			System.out.println("Bidder: "+bidderCountry.userID+" has null country");
        			
        			//---Attach this user to the vector of userCountry.
        			userCountryVector.add(bidderCountry);
        		}
        		
        		/////////////////////// Fill in bidder's rating ///////////////////////////
        		UserRatingSchema bidderRating = new UserRatingSchema();
        		
        		//Bidder's ID again.
        		bidderRating.userID = bidder.getAttributeNode("UserID").getValue();
        		
        		//Bidder's rating.
        		bidderRating.rating = bidder.getAttributeNode("Rating").getValue();
        		
        		//Check for null values to send a warning.
        		if(bidderRating.userID.length()==0 || bidderRating.rating.length()==0)
        			System.out.println("Bidder: "+bidderRating.userID+" has null rating");
        		
        		//---Attach new entry in the user rating vector.
        		userRatingVector.add(bidderRating);
        		
        		/////////////////////// Fill in the bid info schema ///////////////////////
        		BidInfoSchema bidInfo = new BidInfoSchema();		//Individual registry for this bid.
        		
        		//Item ID.
        		bidInfo.itemID = itemSchema.itemID;
        		
        		//Bidder ID.
        		bidInfo.bidderID = bidderRating.userID;
        		
        		//Time of the bid.
        		bidInfo.time = parseDate(getElementTextByTagNameNR(bidNodes[J], "Time"));
        		
        		//Amount of this bid.
        		bidInfo.amount = strip(getElementTextByTagNameNR(bidNodes[J], "Amount"));
        		
        		//Check for null values to send a warning.
        		if(bidInfo.itemID.length()==0 || bidInfo.bidderID.length()==0 || bidInfo.time.length()==0 || bidInfo.amount.length()==0)
        			System.out.println("Bid info: "+bidInfo.itemID+", "+bidInfo.bidderID+", "+bidInfo.time+" has null values");
        		
        		//---Attach this registry to the bid info vector.
        		bidInfoVector.add(bidInfo);
        	}
        	
        	////////// Here, all vectors are ready and need to be translated into arrays/////////
        	itemCategorySchema = new ItemCategorySchema[itemCategoryVector.size()];
        	itemCategoryVector.copyInto(itemCategorySchema);
        	userLocSchema = new UserLocSchema[userLocVector.size()];
        	userLocVector.copyInto(userLocSchema);
        	userCountrySchema = new UserCountrySchema[userCountryVector.size()];
        	userCountryVector.copyInto(userCountrySchema);
        	userRatingSchema = new UserRatingSchema[userRatingVector.size()];
        	userRatingVector.copyInto(userRatingSchema);
        	if(bidInfoVector.size() > 0)							//There were bids?
        	{
        		bidInfoSchema = new BidInfoSchema[bidInfoVector.size()];
        		bidInfoVector.copyInto(bidInfoSchema);
        	}
        	
        	/////////// Verify if the number of bids agree with what was read /////////
        	int nb = Integer.parseInt(itemSchema.numberOfBids);
        	if((bidInfoSchema==null && nb!=0) || (bidInfoSchema!=null && bidInfoSchema.length!=nb))		//There is a problem.
        	{
        		System.out.println("Number of bids read for "+itemSchema.itemID+" does not match with Number_of_Bids field.");
        		System.exit(0);					//End the application.
        	}
        	
        	/////////// Write Files with the collected schema data ////////////
        	writeFiles(itemSchema, itemCategorySchema, itemBuyPriceSchema, userLocSchema, userCountrySchema, userRatingSchema, bidInfoSchema);
        }
        
        ////////////////// Print a signal of how many records were read ///////////////////
        System.out.println("> " + I + " auction records read out of " + itemNodes.length);
        
    }
    
    /* Function to parse the xml date format into TIMESTAMP MySQL format */
    static String parseDate(String xmlDateStr)
    {
    	SimpleDateFormat xmlDateFormat = new SimpleDateFormat("MMM-dd-yy HH:mm:ss");
    	SimpleDateFormat timestampFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    	
    	Date xmlDate = new Date();
    	try
    	{
    		xmlDate = xmlDateFormat.parse(xmlDateStr);		//Create a Date object.
    	}
    	catch(ParseException e)
    	{
    		System.out.println("Failed to parse xml date");
    	}
    	return timestampFormat.format(xmlDate);					//Return the equivalence in timestamp.
    }
    
    /* Function to create all necessary file writer objects */
    static void createFileWriters()
    {
    	try
    	{
    		itemSchemaFile = new PrintWriter(new FileWriter("itemRelation.txt"));
    		userLocSchemaFile = new PrintWriter(new FileWriter("userLocRelation.txt"));
    		userCountrySchemaFile = new PrintWriter(new FileWriter("userCountryRelation.txt"));
    	    itemCategorySchemaFile = new PrintWriter(new FileWriter("itemCategoryRelation.txt"));
    	    userRatingSchemaFile = new PrintWriter(new FileWriter("userRatingRelation.txt"));
    	    itemBuyPriceSchemaFile = new PrintWriter(new FileWriter("itemBuyPriceRelation.txt"));
    	    bidInfoSchemaFile = new PrintWriter(new FileWriter("bidInfoRelation.txt"));
    	}
    	catch(IOException e)
    	{
    		System.out.println("Failed to create data files for the XML tree: " + e.getMessage());
    		System.exit(0);
    	}
    }
    
    /* Function to write data into their respective relation file */
    static void writeFiles(ItemSchema iS, ItemCategorySchema[] icS, ItemBuyPriceSchema ibpS, UserLocSchema[] uS, UserCountrySchema[] ucS, UserRatingSchema[] urS, BidInfoSchema[] biS)
    {	
    	itemSchemaFile.println(iS.toString());				//Write the item information.
    	
    	for(int I=0; I<icS.length; I++)						//Write the item-category relation.
    		itemCategorySchemaFile.println(icS[I].toString());	
    	
    	if(ibpS != null)									//Write the buy price if there is one.
    		itemBuyPriceSchemaFile.println(ibpS.toString());
    	
    	for(int I=0; I<uS.length; I++)						//Write the user location relation.
    		userLocSchemaFile.println(uS[I].toString());
    	
    	for(int I=0; I<ucS.length; I++)						//Write the user country relation.
    		userCountrySchemaFile.println(ucS[I].toString());
    	
    	for(int I=0; I<urS.length; I++)						//Write the user rating relation.
    		userRatingSchemaFile.println(urS[I].toString());
    	
    	for(int I=0; biS!=null && I<biS.length; I++)		//Write the bid info relation.
    		bidInfoSchemaFile.println(biS[I].toString());
    }
    
    /* Function to close all data files */
    static void closeFileWriters()
    {
    	itemSchemaFile.close();				//Attempt to close all open files.
    	userLocSchemaFile.close();
    	itemCategorySchemaFile.close();
    	userCountrySchemaFile.close();
    	userRatingSchemaFile.close();
        itemBuyPriceSchemaFile.close();
        bidInfoSchemaFile.close();
    }
    
    public static void main (String[] args) {
        if (args.length == 0) {
            System.out.println("Usage: java MyParser [file] [file] ...");
            System.exit(1);
        }
        
        /* Initialize parser. */
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setIgnoringElementContentWhitespace(true);      
            builder = factory.newDocumentBuilder();
            builder.setErrorHandler(new MyErrorHandler());
        }
        catch (FactoryConfigurationError e) {
            System.out.println("unable to get a document builder factory");
            System.exit(2);
        } 
        catch (ParserConfigurationException e) {
            System.out.println("parser was unable to be configured");
            System.exit(2);
        }
        
        //Creation of files for writing data.
        createFileWriters();
        
        /* Process all files listed on command line. */
        for (int i = 0; i < args.length; i++) {
            File currentFile = new File(args[i]);
            processFile(currentFile);
        }
        
        //After all files have been processed, close them.
        closeFileWriters();
    }
    
    /* Additional simple classes (as structures) to store the relational schema of tables */
    static class ItemSchema
    {
    	public String itemID;
    	public String name;
    	public String description;
    	public String firstBid;
    	public String currently;
    	public String started;
    	public String ends;
    	public String sellerID;
    	public String numberOfBids;
    	
    	public String toString()	//Overwrite the function toString to return the atts separated by commas.
    	{
    		return itemID+"\t"+name+"\t"+description+"\t"+
    			firstBid+"\t"+currently+"\t"+started+"\t"+ends+
    			"\t"+sellerID+"\t"+numberOfBids;
    	}
    }
    
    static class ItemBuyPriceSchema
    {
    	public String itemID;
    	public String buyPrice;
    	
    	public String toString()	//Overwrite the function toString to return the atts separated by commas.
    	{
    		return itemID+"\t"+buyPrice;
    	}
    }
    
    static class ItemCategorySchema
    {
    	public String itemID;
    	public String category;
    	
    	public String toString()	//Overwrite the function toString to return the atts separated by commas.
    	{
    		return itemID+"\t"+category;
    	}
    }
    
    static class UserLocSchema
    {
    	public String userID;
    	public String location;
    	
    	public String toString()	//Overwrite the function toString to return the atts separated by commas.
    	{
    		return userID+"\t"+location;
    	}
    }
    
    static class UserCountrySchema
    {
    	public String userID;
    	public String country;
    	
    	public String toString()	//Overwrite the function toString to return the atts separated by commas.
    	{
    		return userID+"\t"+country;
    	}
    }
    
    static class UserRatingSchema
    {
    	public String userID;
    	public String rating;
    	
    	public String toString()	//Overwrite the function toString to return the atts separated by commas.
    	{
    		return userID+"\t"+rating;
    	}
    }
    
    static class BidInfoSchema
    {
    	public String itemID;
    	public String bidderID;
    	public String time;
    	public String amount;
    	
    	public String toString()	//Overwrite the function toString to return the atts separated by commas.
    	{
    		return itemID+"\t"+bidderID+"\t"+time+"\t"+amount;
    	}
    }
}
