/* CS144
 *
 * Parser skeleton for processing item-???.xml files. Must be compiled in
 * JDK 1.5 or above.
 *
 * Instructions:
 *
 * This program processes all files passed on the command line (to parse
 * an entire diectory, type "java MyParser myFiles/*.xml" at the shell).
 *
 * At the point noted below, an individual XML file has been parsed into a
 * DOM Document node. You should fill in code to process the node. Java's
 * interface for the Document Object Model (DOM) is in package
 * org.w3c.dom. The documentation is available online at
 *
 * http://java.sun.com/j2se/1.5.0/docs/api/index.html
 *
 * A tutorial of Java's XML Parsing can be found at:
 *
 * http://java.sun.com/webservices/jaxp/
 *
 * Some auxiliary methods have been written for you. You may find them
 * useful.
 */

package edu.ucla.cs.cs144;

import javax.xml.parsers.*;
import org.xml.sax.InputSource;
import org.w3c.dom.*;
import java.io.*;
import java.util.*;

class MyParser 
{   
    static final String[] typeName = {"none","Element","Attr","Text","CDATA","EntityRef","Entity","ProcInstr","Comment","Document","DocType","DocFragment","Notation"};
    
    /* Non-recursive (NR) version of Node.getElementsByTagName(...)
     */
    private Element[] getElementsByTagNameNR(Element e, String tagName) {
        Vector< Element > elements = new Vector< Element >();
        Node child = e.getFirstChild();
        while (child != null) {
            if (child instanceof Element && child.getNodeName().equals(tagName))
            {
                elements.add( (Element)child );
            }
            child = child.getNextSibling();
        }
        Element[] result = new Element[elements.size()];
        elements.copyInto(result);
        return result;
    }
    
    /* Returns the first subelement of e matching the given tagName, or
     * null if one does not exist. NR means Non-Recursive.
     */
    private Element getElementByTagNameNR(Element e, String tagName) {
        Node child = e.getFirstChild();
        while (child != null) {
            if (child instanceof Element && child.getNodeName().equals(tagName))
                return (Element) child;
            child = child.getNextSibling();
        }
        return null;
    }
    
    /* Returns the text associated with the given element (which must have
     * type #PCDATA) as child, or "" if it contains no text.
     */
    private String getElementText(Element e) {
        if (e.getChildNodes().getLength() == 1) {
            Text elementText = (Text) e.getFirstChild();
            return elementText.getNodeValue();
        }
        else
            return "";
    }
    
    /* Returns the text (#PCDATA) associated with the first subelement X
     * of e with the given tagName. If no such X exists or X contains no
     * text, "" is returned. NR means Non-Recursive.
     */
    private String getElementTextByTagNameNR(Element e, String tagName) {
        Element elem = getElementByTagNameNR(e, tagName);
        if (elem != null)
            return getElementText(elem);
        else
            return "";
    }
    
    /* Process the XML string that represents a given item.
     */
    public ItemBean processXMLString(String xmlString) 
    {
    	Document doc = null;
    	/* Initialize the parser */
    	try 
    	{
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            InputSource is = new InputSource();
            is.setCharacterStream(new StringReader(xmlString));

            doc = db.parse(is);
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        /**************************** Data processing starts ******************************/
        ItemBean itemBean = new ItemBean();				//Create the bean to return.
        Element itemNode = doc.getDocumentElement();	//Get the root item node.
        	
    	////////////Create the schemas and vectors for data that will be encountered//////////
    	Vector <String> categoriesVector = new Vector <String>();
    	Vector <Bid> bidsVector = new Vector <Bid>();
    	//////////////////////////////////////////////////////////////////////////////////////
    	
    	//Item's ID.
    	Attr attr = itemNode.getAttributeNode("ItemID");
    	itemBean.setItemID(attr.getValue());
    	
    	//Item's name.
    	itemBean.setName(getElementTextByTagNameNR(itemNode, "Name"));
    	
    	//Item's description.
    	String desc = getElementTextByTagNameNR(itemNode, "Description");
    	if(desc.length() > 4000)		//Check length to not exceed 4000 chars.
    		desc = desc.substring(0, 4000);
    	itemBean.setDescription(desc);
    	
    	//Item's first bid.
    	itemBean.setFirstBid(getElementTextByTagNameNR(itemNode, "First_Bid"));
    	
    	//Item's current value.
    	itemBean.setCurrently(getElementTextByTagNameNR(itemNode, "Currently"));
    	
    	//Item's auction start time.
    	itemBean.setStarted(getElementTextByTagNameNR(itemNode, "Started"));
    	
    	//Item's auction ends time.
    	itemBean.setEnds(getElementTextByTagNameNR(itemNode, "Ends"));
    	
    	//Item's number of bids.
    	itemBean.setNumberOfBids(getElementTextByTagNameNR(itemNode, "Number_of_Bids"));
    	
    	User seller = new User();
    	
    	//Item's seller.
    	seller.setUserID(getElementByTagNameNR(itemNode, "Seller").getAttributeNode("UserID").getValue());

    	//Item's seller's location.
    	seller.setLocation(getElementTextByTagNameNR(itemNode, "Location"));
    	
    	//Item's seller's country.
    	seller.setCountry(getElementTextByTagNameNR(itemNode, "Country"));
    	
    	//Item's seller's rating.
    	seller.setRating(getElementByTagNameNR(itemNode, "Seller").getAttributeNode("Rating").getValue());
    	
    	itemBean.setSeller(seller);
    	
    	//Item's buy price.
    	itemBean.setBuyPrice(getElementTextByTagNameNR(itemNode, "Buy_Price"));
    	
    	///////////////////////// Fill in the categories of the item /////////////////////////
    	Element[] categoryNodes = getElementsByTagNameNR(itemNode, "Category");
    	for(int J=0; J < categoryNodes.length; J++)
    	{
    		categoriesVector.add(getElementText(categoryNodes[J]));
    	}
    	
    	//////////////////////// Fill in data about the bids ///////////////////////
    	Element bidsNode = getElementByTagNameNR(itemNode, "Bids");		//New root.
    	Element[] bidNodes = getElementsByTagNameNR(bidsNode, "Bid");
    	for(int J=0; J < bidNodes.length; J++)
    	{
    		Bid bidInfo = new Bid();			//Temporal object for bid info.
    		User bidder = new User();			//Temporal object for the bidder.
    		
    		//////////////////////// Fill in bidder's data //////////////////////
    		Element bidderNode = getElementByTagNameNR(bidNodes[J], "Bidder");	//Get a reference.
    		
    		//Bidder's id.
    		bidder.setUserID(bidderNode.getAttributeNode("UserID").getValue());
    		
    		//Bidder's location.
        	bidder.setLocation(getElementTextByTagNameNR(bidderNode, "Location"));
        	
        	//Bidder's country.
        	bidder.setCountry(getElementTextByTagNameNR(bidderNode, "Country"));
        	
        	//Bidder's rating.
        	bidder.setRating(bidderNode.getAttributeNode("Rating").getValue());
    		
    		/////////////////////// Fill in the bid info ///////////////////////
    		bidInfo.setBidder(bidder);
    		
    		//Time of the bid.
    		bidInfo.setTime(getElementTextByTagNameNR(bidNodes[J], "Time"));
    		
    		//Amount of this bid.
    		bidInfo.setAmount(getElementTextByTagNameNR(bidNodes[J], "Amount"));
    		
    		//Attach this registry to the bid info vector.
    		bidsVector.add(bidInfo);
        }
    	
    	Bid[] bids = new Bid[bidsVector.size()];					//Create the array attached to bids.
    	String[] categories = new String[categoriesVector.size()];	//Create the array attached to categories.
    	
    	bidsVector.copyInto(bids);
    	categoriesVector.copyInto(categories);
    	
    	itemBean.setCategories(categories);
    	itemBean.setBids(bids);
        	
    	/////////// Verify if the number of bids agree with what was read /////////
    	int nb = Integer.parseInt(itemBean.getNumberOfBids());
    	if((bids.length==0 && nb!=0) || (bids.length>0 && bids.length!=nb))	//There is a problem.
    	{
    		System.err.println("Number of bids read for "+bids.length+" does not match with Number_of_Bids field.");
    		System.exit(2);					//End the application.
    	}
    	
    	return itemBean;					//Return a bean with information.
    }
}
